/*
 * Copyright (C) 2003  Stefan Armbruster [stefan@armbruster-it.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: IterativeMapModel.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.sam.models;

import java.util.Map;

import org.apache.log4j.Logger;

/** iterates over a Map, uses the special keys "Key" and "Value" for returning key-value-pairs
 * @author Stefan Armbruster
 * @version $Id: IterativeMapModel.java 114 2005-12-09 15:51:51Z christianc $
 */
public class IterativeMapModel extends IteratorModel {

    protected static Logger logger = Logger.getLogger(IterativeMapModel.class.getName());

    protected Map map;

    /** initialize the model
     * @param name Name of the model
     * @param map Map to iterate
     */
    public IterativeMapModel(String name, Map map) {
        super(name, map.entrySet().iterator());
        this.map = map;
    }

    public IterativeMapModel(String name) {
        super(name);
    }

    public Object getItem(String key) {

        Map.Entry entry = (Map.Entry)_current;
        logger.debug("trying to get key " + key + " from " + map);
        if (key.equals("Key")) {
            return (entry!=null)? entry.getKey() : null;
        } else if (key.equals("Value")) {
            return (entry!=null)? entry.getValue() : null;
        } else {
            //return super.getItem(key);
            return map.get(key);
        }
    }
    }

