/*
 * Copyright (C) 2003  Stefan Armbruster [stefan@armbruster-it.de]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: ModifyDomViewHandler.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.contrib.sam.xmlform;

import java.io.IOException;
import java.util.List;

import javax.servlet.ServletException;

import org.apache.log4j.Logger;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.BTemplate;
import org.barracudamvc.core.comp.DefaultTemplateView;
import org.barracudamvc.core.comp.TemplateModel;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.event.EventException;
import org.barracudamvc.core.event.helper.DefaultViewHandler;
import org.barracudamvc.core.util.dom.DefaultDOMLoader;
import org.w3c.dom.Document;

/** Pretty similar class to {@link org.barracudamvc.core.event.helper.BTemplateViewHandler
 * }. The main differences are
 * <ul>
 * <li>A <CODE>ViewContext</CODE> is passed to {@link #getTemplateModels } and
 * {@link #getTemplateClass}
 * <li>provides a hook for modifying a XMLC class <B>before</B> the whole
 * Model/Component rendering is performed
 * </ul>
 * @author Stefan Armbruster
 * @version $Id: ModifyDomViewHandler.java 114 2005-12-09 15:51:51Z christianc $
 */
public abstract class ModifyDomViewHandler extends DefaultViewHandler {

    protected static Logger logger = Logger.getLogger(ModifyDomViewHandler.class.getName());

    protected BTemplate templateComp = null;

    /** abstract method for getting als required models
     * @param vc the ViewContext of the current RenderEvent
     * @throws EventException
     * @throws ServletException
     * @return either a single model or a {@link java.util.List} of models
     */
    public abstract Object getTemplateModels(ViewContext vc) throws EventException, ServletException, IOException;

    /** Return an instance of the template class (must implement Document) */
    public abstract Class getTemplateClass(ViewContext vc) throws EventException, ServletException, IOException;

    /**
     * Provide a handle to the underlying BTemplate component
     */
    public BTemplate getBTemplate() {
        return templateComp;
    }

    /** Placeholder for DOM-modifications prior to directive mappings. The default
     * implementation is empty.
     * @param page a instance of the class returned by {@link #getTemplateClass}
     * @param vc the context of the current event
     */
    protected void modifyDOM(Document page, ViewContext vc) throws EventException, ServletException, IOException {
    }

    /**
     * Generate the view
     */
    public Document handleViewEvent(BComponent root) throws EventException, ServletException, IOException {

        ViewContext vc = getViewContext();
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(getTemplateClass(vc),  vc.getViewCapabilities().getClientLocale());
        modifyDOM(page, vc);

        //create a template component and bind it to the views
        templateComp = new BTemplate();
        Object o = getTemplateModels(vc);
        if (o instanceof TemplateModel) templateComp.addModel((TemplateModel) o);
        else if (o instanceof List) templateComp.addModels((List) o);
        else if (o == null) { }
        else throw new RuntimeException("Fatal err: Model must return either TemplateModel or List");
        templateComp.setView(new DefaultTemplateView(page.getDocumentElement()));
        root.addChild(templateComp);

        return page;
    }
}
