/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: GlobalUtilities.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.examples;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletResponse;

import org.barracudamvc.core.event.ApplicationGateway;
import org.barracudamvc.core.event.BaseEvent;
import org.barracudamvc.core.event.BaseEventListener;
import org.barracudamvc.core.event.DefaultBaseEvent;
import org.barracudamvc.core.event.DefaultBaseEventListener;
import org.barracudamvc.core.event.DefaultEventGateway;
import org.barracudamvc.core.event.DefaultListenerFactory;
import org.barracudamvc.core.event.EventException;
import org.barracudamvc.core.event.HttpResponseEvent;
import org.barracudamvc.core.event.ListenerFactory;
import org.barracudamvc.core.event.ViewEventContext;


/**
 * Global Utilities. This class is primarily responsible for 
 * handling an HttpResponseEvent and generating a "Resource
 * not found error"
 */
public class GlobalUtilities extends DefaultEventGateway {

    //public constants

    //this defines the various event handlers
    private ListenerFactory httpResponseFactory =         new DefaultListenerFactory() {public BaseEventListener getInstance() {return new HttpResponseHandler();}        public String getListenerID() {return getID(HttpResponseHandler.class);}};                

    /**
     * Public constructor
     */
    public GlobalUtilities() {
        //specify who's interested in what
        specifyLocalEventInterests(httpResponseFactory, HttpResponseEvent.class);
    }

    //------------------------------------------------------------
    //                Model 2 - View Event Handlers
    //------------------------------------------------------------
    /**
     * HttpResponseHandler - this is the global handler for HttpResponseEvents.
     * If we get one of these events it means that something very unusual happened
     * in the system (like somebody requested an invalid event or something). So,
     * display a simple page indicating that the resource was not available.
     */
    class HttpResponseHandler extends DefaultBaseEventListener {
        public void handleViewEvent(ViewEventContext context) throws EventException, ServletException, IOException {
            //unpack the necessary entities from the context
            BaseEvent event = context.getEvent();
            HttpServletResponse resp = context.getResponse();

            //csc_061202.1 - added    
            //set the caching hdrs (this will allow the page to be cached by the browser)
            resp.setHeader("Cache-Control","max-age=0");
            resp.setDateHeader("Last-Modified", System.currentTimeMillis());

            //render the page
            resp.setContentType("text/html");
            PrintWriter out = resp.getWriter();
            out.println("<html><head><title>Resource not Found</title></head>");
            out.println("<body><font face=\"Arial\">");

            //display the general error
            String targetEventName = (String) context.getState(ApplicationGateway.TARGET_EVENT_NAME);
            out.println("<p>Beep-beep-beep! We're sorry your call did not go through. Will you please hang up and try your call again...");
            out.println("<p>If you received this message, it indicates that either <strong>"+targetEventName+"</strong> is not a valid event OR the system is misconfigured. If you feel there is a problem with the system please <a href=\"mailto:christianc@lutris.com\">contact the application administrator</a>... ");

            //log the specific problem
            BaseEvent be = DefaultBaseEvent.getRootEvent(event);
            out.println("<p>The base event that triggered this action was:"+be+"<br>");
            out.println("Source:"+be.getSource()+"<br>");
            
            //footer
            out.println("</font></body></html>");
        }
    }
}

