package org.barracudamvc.examples.xmlc;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.barracudamvc.core.comp.AbstractTableModel;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.BList;
import org.barracudamvc.core.comp.BTable;
import org.barracudamvc.core.comp.BText;
import org.barracudamvc.core.comp.DefaultListModel;
import org.barracudamvc.core.comp.DefaultTableView;
import org.barracudamvc.core.comp.DefaultView;
import org.barracudamvc.core.comp.ListModel;
import org.barracudamvc.core.comp.TableModel;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.comp.helper.ComponentGateway;
import org.barracudamvc.core.util.dom.DOMUtil;
import org.barracudamvc.core.util.dom.DOMWriter;
import org.barracudamvc.core.util.dom.DefaultDOMLoader;
import org.barracudamvc.core.util.dom.DefaultDOMWriter;
import org.barracudamvc.plankton.data.*;
import org.barracudamvc.plankton.http.SessionServices;
import org.w3c.dom.*;

/**
 * Here's an example of how the component stuff can be implemented
 * in a standalone servlet.
 */
public class CompEx1 extends ComponentGateway {

    private static String SAMPLE_SCREEN = CompEx1.class+".SampleScreen";

    static final String TEMPLATE_PAGE = "TemplatePage";


    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //see if the component tree is in the session. If not, create it
        //and place it there
        SampleScreen screen = null;
        HttpSession session = SessionServices.getSession(req, false);
        if (session!=null) screen = (SampleScreen) session.getAttribute(SAMPLE_SCREEN);
        if (screen==null) {
            screen = new SampleScreen();
            if (session!=null) session.setAttribute(SAMPLE_SCREEN, screen);
        }

        //now, update the data/models
        screen.getText().setText("[Dynamic text]");
        DefaultListModel listModel = screen.getListModel();
        if (listModel.size()>=15) listModel.clear();    //this will cause the lists that get repopulated to grow everytime you refresh the page, so you can see the thing changing. Once it gets big enough, truncate.
        int cntr = listModel.size();
        listModel.add("[Dynamic list item "+(++cntr)+"]");
        listModel.add("[Dynamic list item "+(++cntr)+"]");
        listModel.add("[Dynamic list item "+(++cntr)+"]");
        listModel.add("[Dynamic list item "+(++cntr)+"]");
        listModel.add("[Dynamic list item "+(++cntr)+"]");
        root.addChild(screen.getRoot());
            

        //return the page
        return screen.getPage();
    }

    /**
     * <p>Get a DOMWriter. By default, we use a DefaultDOMWriter. If
     * you'd like to use something else, override this method.
     *
     * @return a DOMWriter to be used to render the DOM
     */
    public DOMWriter getDOMWriter() {
        //return DefaultDOM writer
        return new DefaultDOMWriter(false);
    }



    class SampleScreen {

        CompEx1HTML page = null;
        BComponent wcRoot = null;
        BText wcText = null;
        DefaultListModel listModel1 = null;

        public SampleScreen() throws IOException {
            //load the localized DOM template
            page = (CompEx1HTML) DefaultDOMLoader.getGlobalInstance().getDOM(CompEx1HTML.class);

            //csc_012605_1 - added
            //lets save a second copy of the page which will act as our template (this is terribly 
            //inefficient; the only reason we are doing it this way is since we know longer have
            //the element factory. This is NOT an example of how you should do it in your apps)
            ObjectRepository.getLocalRepository().putState(TEMPLATE_PAGE, DefaultDOMLoader.getGlobalInstance().getDOM(CompEx1HTML.class));

            //now build our component hierarchy (in many cases, fairly flat)
            //   and bind the components to the views
            wcRoot = new BComponent();
            wcRoot.setName("Root");

            //...Text
            wcText = new BText();
            wcText.setName("Text1");
            wcText.addView(new DefaultView("View0",page.getElementText0()));
            wcText.addView(new DefaultView("View1",page.getElementText1()));
            wcText.addView(new DefaultView("View2",page.getElementText2()));
            wcText.addView(new DefaultView("View3",page.getElementText3()));
            wcText.addView(new DefaultView("View4",page.getElementText4()));
            wcText.addView(new DefaultView("View5",page.getElementText5()));
            wcText.addView(new DefaultView("View6",page.getElementText6()));
            wcText.addView(new DefaultView("View7",page.getElementText7()));
            wcText.addView(new DefaultView("View8",page.getElementText8()));
            wcText.addView(new DefaultView("View9",page.getElementText9()));
            wcText.addView(new DefaultView("View10",page.getElementText10()));
            wcText.addView(new DefaultView("View11",page.getElementText11()));
            wcText.addView(new DefaultView("View12",page.getElementText12()));
            wcText.addView(new DefaultView("View13",page.getElementText13()));
            wcText.addView(new DefaultView("View14",page.getElementText14()));
            wcText.addView(new DefaultView("View15",page.getElementText15()));
            wcText.addView(new DefaultView("View16",page.getElementText16()));
            wcText.addView(new DefaultView("View17",page.getElementText17()));
            wcText.addView(new DefaultView("View18",page.getElementText18()));
            wcText.addView(new DefaultView("View19",page.getElementText19()));
            wcText.addView(new DefaultView("View20",page.getElementText20()));
            wcText.addView(new DefaultView("View21",page.getElementText21()));
            wcText.addView(new DefaultView("View22",page.getElementText22()));
            wcText.addView(new DefaultView("View23",page.getElementText23()));
            wcText.addView(new DefaultView("View24",page.getElementText24()));
            wcText.addView(new DefaultView("View25",page.getElementText25()));
            wcText.addView(new DefaultView("View26",page.getElementText26()));
            wcText.addView(new DefaultView("View27",page.getElementText27()));
            wcText.addView(new DefaultView("View28",page.getElementText28()));
            wcText.addView(new DefaultView("View29",page.getElementText29()));
            wcText.addView(new DefaultView("View30",page.getElementText30()));
            wcText.addView(new DefaultView("View31",page.getElementText31()));
            wcRoot.addChild(wcText);
//wcText.setVisible(false);
//wcText.setEnabled(false);

            //...Lists
            BText bt1 = new BText("[Dynamic list item XX]");
            bt1.setView(new DefaultView(page.getElementDynamicListText()));
            wcRoot.addChild(bt1);

            listModel1 = new DefaultListModel();
            BList wcList1 = new BList(listModel1);
            wcList1.setName("List1");
            wcList1.addView(new DefaultView(page.getElementList1()));
            wcList1.addView(new DefaultView(page.getElementList2()));
            wcList1.addView(new DefaultView(page.getElementList3()));
            wcList1.addView(new DefaultView(page.getElementList4()));
            wcList1.addView(new DefaultView(page.getElementList5()));
            wcRoot.addChild(wcList1);
           
            ListModel listModel2 = new CustomListModel2();
            BList wcList2 = new BList(listModel2);
            wcList2.setName("List2");
            wcList2.addView(new DefaultView(page.getElementDefList()));
            wcRoot.addChild(wcList2);

            ListModel listModel3 = new CustomListModel3();
            BList wcList3 = new BList(listModel3);
            wcList3.setName("List3");
            wcList3.addView(new DefaultView(page.getElementOptGroupList()));
            wcRoot.addChild(wcList3);
            
            ListModel listModel4 = new CustomListModel4();
            BList wcList4 = new BList(listModel4);
            wcList4.setName("List4");
            wcList4.addView(new DefaultView(page.getElementList6()));
            wcRoot.addChild(wcList4);
            
            ListModel listModel5 = new CustomListModel5();
            BList wcList5 = new BList(listModel5);
            wcList5.setName("List5");
            wcList5.addView(new DefaultView(page.getElementList7()));
            wcRoot.addChild(wcList5);

//wcList1.setEnabled(false);
//wcList2.setEnabled(false);
//wcList3.setEnabled(false);
//wcList4.setEnabled(false);
//wcList5.setEnabled(false);

            //..Tables
            TableModel tableModel1 = new CustomTableModel1();
            BTable wcTable1 = new BTable(tableModel1);
            wcTable1.setName("Table1");
            wcTable1.addView(new DefaultTableView(page.getElementTable1()));
            wcRoot.addChild(wcTable1);

            TableModel tableModel2 = new CustomTableModel2();
            BTable wcTable2 = new BTable(tableModel2);
            wcTable2.setName("Table2");
            wcTable2.addView(new DefaultTableView(page.getElementTable2()));
            wcRoot.addChild(wcTable2);

            TableModel tableModel3 = new CustomTableModel1();
            TableModel tableHdrModel3 = new CustomHeaderModel1();
            TableModel tableFtrModel3 = new CustomFooterModel1();
//            BTable wcTable3 = new BTable(tableHdrModel3, tableModel3, tableFtrModel3);
            BTable wcTable3 = new BTable();
            wcTable3.setHeaderModel(tableHdrModel3);
            wcTable3.setModel(tableModel3);
            wcTable3.setFooterModel(tableFtrModel3);
            wcTable3.setName("Table3");
            wcTable3.addView(new DefaultTableView(page.getElementTable3()));
            wcRoot.addChild(wcTable3);
        }
        
        public CompEx1HTML getPage() {return page;}
        public BComponent getRoot() {return wcRoot;}
        public BText getText() {return wcText;}
        public DefaultListModel getListModel() {return listModel1;}

    }

    //csc_012605_1 - added
    public static Node getTemplateNode(ViewContext vc, String id) {
        Document templatePage = (Document) ObjectRepository.getLocalRepository().getState(TEMPLATE_PAGE);
        return vc.getDocument().importNode(templatePage.getElementById(id), true);    
    }
    
    /**
     * demonstrate a simple table model that just passes
     * back strings. The HTML table uses <thead>,<tbody>, and <tfoot>
     * tags, so all we have to pass back is the data. Cool.
     */
    class CustomTableModel1 extends AbstractTableModel {
        public Object getItemAt(int row, int col) {
            return ""+(row*col);        
        }
        public int getColumnCount() {return 4;}
        public int getRowCount() {return 5;}
    }

    /**
     * demonstrate how we can implement the header portion as a table
     * model on its own
     */
    class CustomHeaderModel1 extends AbstractTableModel {
        public Object getItemAt(int row, int col) {
            ViewContext vc = getViewContext();
            try {
//csc_012605_1                ElementFactory ef = vc.getElementFactory();
//csc_012605_1                Node node = ef.getElement("Table3Header").cloneNode(true);
                Node node = getTemplateNode(vc, "Table3Header");    //csc_012605_1
                BText bt = new BText("Sample Header "+row);
                bt.setView(new DefaultView(node));
                return bt;    
            } catch (Exception e) {
                return "Dflt Sample Header";
            }
        }
        public int getColumnCount() {return 1;}
        public int getRowCount() {return 3;}
    }
    
    /**
     * demonstrate how we can implement the footer portion as a table
     * model on its own
     */
    class CustomFooterModel1 extends AbstractTableModel {
        public Object getItemAt(int row, int col) {
            ViewContext vc = getViewContext();
            try {
//csc_012605_1                ElementFactory ef = vc.getElementFactory();
//csc_012605_1                Node node = ef.getElement("Table3Footer").cloneNode(true);
                Node node = getTemplateNode(vc, "Table3Footer");    //csc_012605_1
                BText bt = new BText("Sample Footer "+row);
                bt.setView(new DefaultView(node));
                return bt;
            } catch (Exception e) {
                return "Dflt Sample Footer";
            }
        }
        public int getColumnCount() {return 1;}
        public int getRowCount() {return 2;}
    }
    
    /**
     * demonstrate a simple table model that just passes
     * back strings. The HTML table simpler in that it doesn't
     * have a <tbody> structure but this in turn means we have
     * to provide those in addition to the data...
     */
    class CustomTableModel2 extends AbstractTableModel {
    
        int rowCnt = 5;
        int hdrCnt = 2;
        int ftrCnt = 1;
    
        public Object getItemAt(int row, int col) {
            ViewContext vc = getViewContext();
            //header 0
            if (row==0) {
                if (col>0) return null; //the node returned will span all the columns, so only return data for the first col
                try {
//csc_012605_1                    ElementFactory ef = vc.getElementFactory();
//csc_012605_1                    Node node = ef.getElement("Table2HdrText").cloneNode(true);
                    Node node = getTemplateNode(vc, "Table2HdrText");    //csc_012605_1
                    BText bt = new BText("Table B: (everything is &lt;tr&gt; tags)");
                    bt.setView(new DefaultView(node));
                    return bt;
                } catch (Exception e) {
                    return "Dflt Hdr 0";
                }

            //header 1
            } else if (row==1) {
                try {
//csc_012605_1                    ElementFactory ef = vc.getElementFactory();
//csc_012605_1                    Node node = ef.getElement("Table2HdrCell").cloneNode(true);
                    Node node = getTemplateNode(vc, "Table2HdrCell");    //csc_012605_1
                    BText bt = new BText("Column "+col);
                    bt.setView(new DefaultView(node));
                    return bt;
                } catch (Exception e) {
                    return "Hdr Col "+col;
                }
                
            //footer 0
            } else if (row==(getRowCount()-1)) {
                 if (col>0) return null; //the node returned will span all the columns, so only return data for the first col
                try {
//csc_012605_1                    ElementFactory ef = vc.getElementFactory();
//csc_012605_1                    Node node = ef.getElement("Table2FtrList").cloneNode(true);
                    Node node = getTemplateNode(vc, "Table2FtrList");    //csc_012605_1
                    BText bt = new BText("Look ma! Ye can see me footer!");
                    bt.setView(new DefaultView(node));
                    return bt;
                } catch (Exception e) {
                    return "Dflt Ftr 0";
                }

            //data
            } else {
                return ""+((row-hdrCnt)*col);        
            }
        }
        
        public int getColumnCount() {return 4;}
        public int getRowCount() {return rowCnt+hdrCnt+ftrCnt;}
    }

    /**
     * this is really kind of hokey since the data is completely
     * canned, but it demonstrates
     *  a) how you can write an inner class that implements your model
     *  b) how you can pass back an array of objects and they will
     *     all get added to the list
     */
    class CustomListModel2 extends DefaultListModel {
        public Object getItemAt(int index) {
            ViewContext vc = getViewContext();
//csc_012605_1            ElementFactory ef = vc.getElementFactory();
//csc_012605_1            Element title = (Element) (ef.getElement("DT").cloneNode(true));
            Element title = (Element) vc.getDocument().createElement("DT");    //csc_012605_1
            title.removeAttribute("id");
            DOMUtil.setTextInNode(title, "[Dynamic title "+(index+1)+"]", false);
//csc_012605_1            Element def = (Element) (ef.getElement("DD").cloneNode(true));
            Element def = (Element) vc.getDocument().createElement("DD");    //csc_012605_1
            def.removeAttribute("id");
            DOMUtil.setTextInNode(def, "[Dynamic def "+(index+1)+"]", false);
            return new Object[] {title, def};
        }

        public int getSize() {return 3;}
    }


    /**
     * this is also kind of hokey, but it demonstrates how you can pass back
     * nested BComponents...(we're operating against a SELECT node here)
     */
    class CustomListModel3 extends DefaultListModel {
        public Object getItemAt(int index) {
            ViewContext vc = getViewContext();
//csc_072604_2            ElementFactory ef = vc.getElementFactory();
        
            //for the first index, pass back a NONE option
            if (index==0) {
//csc_072604_2                Element el = (Element) ef.getDocument().createElement("OPTION");
                Element el = (Element) vc.getDocument().createElement("OPTION");    //csc_072604_2
                DOMUtil.setTextInNode(el, "[Dynamic none]", false);
                return el;
                
            //otherwise pass back a group of options (note: here we return a BComponent)
            } else {
//csc_072604_2                Element el = (Element) ef.getDocument().createElement("OPTGROUP");
                Element el = (Element) vc.getDocument().createElement("OPTGROUP");  //csc_072604_2
                String descr = "Subport A."+index;
                if (index==2) descr = "Subport B."+index;
                else if (index==3) descr = "Subport C."+index;
                BList elList = new BList(new CustomListModel3a(descr, index));
                elList.setView(new DefaultView(el));
                return elList;
            }
        }

        public int getSize() {return 4;}
    }

    class CustomListModel3a extends DefaultListModel {
        String descr = null;
        int max = 1;

        public CustomListModel3a(String idescr, int imax) {
            descr = idescr;
            max = imax;
        }

        public Object getItemAt(int index) {
            ViewContext vc = getViewContext();
//csc_072604_2            ElementFactory ef = vc.getElementFactory();
            List list = new ArrayList();
            for (int i=0; i<max; i++) {
//csc_072604_2                Element el = (Element) ef.getDocument().createElement("OPTION");
                Element el = (Element) vc.getDocument().createElement("OPTION");    //csc_072604_2
                DOMUtil.setTextInNode(el, "[Dynamic "+descr+i+"]", false);
                list.add(el);
            }
            return list;    //note that here we return a list of nodes
        }
        
        public int getSize() {return 1;}
    }


    /**
     * This list model returns a list of complex components (just
     * to demonstrate how its done). In this case we return an option control
     * followed by some text.
     */
    class CustomListModel4 extends DefaultListModel {
    
        public Object getItemAt(int index) {
            ViewContext vc = getViewContext();
//csc_072604_2            ElementFactory ef = vc.getElementFactory();
//csc_072604_2            Element el = (Element) ef.getDocument().createElement("INPUT");
            Element el = (Element) vc.getDocument().createElement("INPUT"); //csc_072604_2
            el.setAttribute("type","radio");
            el.setAttribute("value","value"+index);
            el.setAttribute("name","LM4");
//csc_072604_2            Node textNode = (Node) ef.getDocument().createTextNode("Dyn"+index+" ");
            Node textNode = (Node) vc.getDocument().createTextNode("Dyn"+index+" ");    //csc_072604_2
            return new Object[] {el, textNode};
        }

        public int getSize() {return 3;}
    }

    /**
     * This list model demonstrates how we can return complex
     * snippets of HTML that actually come from other XMLC pages
     * (note we have to do importNode() to make this work)...
     */
    class CustomListModel5 extends DefaultListModel {
    
        public Object getItemAt(int index) {
            ViewContext vc = getViewContext();
//csc_072604_2            ElementFactory ef = vc.getElementFactory();
            Node node = null;
//            if (index==0) node = ((SnippetEx1HTML) xmlcFactory.create(SnippetEx1HTML.class)).getElementSnippet();
//            else if (index==1) node = ((SnippetEx2HTML) xmlcFactory.create(SnippetEx2HTML.class)).getElementSnippet();
//            else node = ((SnippetEx3HTML) xmlcFactory.create(SnippetEx3HTML.class)).getElementSnippet();
            try {
                if (index==0) node = ((SnippetEx1HTML) DefaultDOMLoader.getGlobalInstance().getDOM(SnippetEx1HTML.class)).getElementSnippet();
                else if (index==1) node = ((SnippetEx2HTML) DefaultDOMLoader.getGlobalInstance().getDOM(SnippetEx2HTML.class)).getElementSnippet();
                else node = ((SnippetEx3HTML) DefaultDOMLoader.getGlobalInstance().getDOM(SnippetEx3HTML.class)).getElementSnippet();
//csc_072604_2                node = ef.getDocument().importNode(node, true);
                node = vc.getDocument().importNode(node, true); //csc_072604_2
                return node;
            } catch (IOException e) {
                return "[snippet render failed "+e+"]";
            }
        }

        public int getSize() {return 3;}
        
        public String toString() {
            return "[Error rendering Snippet list]";    //if we see this message in the HTML it indicates there was a problem
        }
    }
}
