package org.barracudamvc.tutorials.comp;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.barracudamvc.core.comp.AbstractTemplateModel;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.BTemplate;
import org.barracudamvc.core.comp.DefaultTemplateView;
import org.barracudamvc.core.comp.TemplateModel;
import org.barracudamvc.core.comp.TemplateView;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.comp.helper.ComponentGateway;
import org.barracudamvc.core.util.dom.DefaultDOMLoader;
import org.barracudamvc.tutorials.xmlc.HelloWorld1HTML;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Here's an example of HelloWorld using the BTemplate
 * component
 */
public class HelloWorld1 extends ComponentGateway {

    //xmlc factory (ok as static because its threadsafe)
//     private static XMLCFactory xmlcFactory = null;

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest. 
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //load the DOM object and find the target node
//        if (xmlcFactory==null) xmlcFactory = new XMLCStdFactory(this.getClass().getClassLoader(), new StreamXMLCLogger());
//        XMLObject page = xmlcFactory.create(HelloWorld1HTML.class);

        //load the localized DOM template
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(HelloWorld1HTML.class, vc.getViewCapabilities().getClientLocale());
        Node node = page.getElementById("HelloWorld");

        //create the component and bind it to the view and the model
        TemplateView tv = new DefaultTemplateView(node);
        TemplateModel tm = new HelloWorldModel();
        BTemplate templateComp = new BTemplate(tm);
        templateComp.setView(tv);

        //add the component to the root
        root.addChild(templateComp);
    
        //return the page
        return page;
    }

    //--------------- HelloWorldModel ------------------------
    /**
     * HelloWorldModel
     */
    class HelloWorldModel extends AbstractTemplateModel {
        //register the model by name
        public String getName() {
            return "HelloWorld";
        }
        
        //provide items by key
        public Object getItem(String key) {
            if (key.equals("Title")) return "Hello World 1 example";
            else if (key.equals("Descr")) return "This example demonstrates the simplest way to use the BTemplate component. Here we are implementing the Model manually as an inner class.";
            else if (key.equals("Hello")) return "Hello World! Hi Ma! Hi Pa!";
            else return super.getItem(key);
        }
    }
}
