/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: HelloWorld3.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.tutorials.comp;

import java.io.IOException;
import java.util.Properties;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.barracudamvc.core.comp.AbstractTemplateModel;
import org.barracudamvc.core.comp.BComponent;
import org.barracudamvc.core.comp.BTemplate;
import org.barracudamvc.core.comp.DefaultTemplateView;
import org.barracudamvc.core.comp.IterativeModel;
import org.barracudamvc.core.comp.TemplateView;
import org.barracudamvc.core.comp.ViewContext;
import org.barracudamvc.core.comp.helper.ComponentGateway;
import org.barracudamvc.core.util.dom.DefaultDOMLoader;
import org.barracudamvc.plankton.data.MapStateMap;
import org.barracudamvc.tutorials.xmlc.HelloWorld3XML;
import org.barracudamvc.tutorials.xmlc.HelloWorld3aXML;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * Here's an example of HelloWorld using the BTemplate
 * component
 */
public class HelloWorld3 extends ComponentGateway {

    //xmlc factory (ok as static because its threadsafe)
//     private static XMLCFactory xmlcFactory = null;

    //-------------------- ComponentGateway ----------------------
    /**
     * Handle the default HttpRequest.
     */
    public Document handleDefault (BComponent root, ViewContext vc, HttpServletRequest req, HttpServletResponse resp) throws IOException {
        //figure out which page we're after by looking for an optional
        //page parameter in the Req
        String pageNo = req.getParameter("page");
        if (pageNo==null) pageNo = "1";
        boolean useIDTemplate = (!pageNo.equals("1"));
        // rtl_20010612 Class pageCl = HelloWorld3HTML.class;
        // rtl_20010612 if (useIDTemplate) pageCl = HelloWorld3aHTML.class;
        Class pageCl = HelloWorld3XML.class;
        if (useIDTemplate) pageCl = HelloWorld3aXML.class;

        //load the DOM object
//        if (xmlcFactory==null) xmlcFactory = new XMLCStdFactory(this.getClass().getClassLoader(), new StreamXMLCLogger());
//        XMLObject page = xmlcFactory.create(pageCl);

        //load the localized DOM template
        Document page = DefaultDOMLoader.getGlobalInstance().getDOM(pageCl, vc.getViewCapabilities().getClientLocale());

        //find the node we're after
        Node node = page.getDocumentElement();

        //create a view on the node
        TemplateView tv = null;
        if (useIDTemplate) {
            String propFileName = "HelloWorld3a.properties";
            Properties props = new Properties();
            try {
                props.load(this.getClass().getResourceAsStream(propFileName));
                tv = new DefaultTemplateView(node, "id", new MapStateMap(props));
            } catch (IOException e) {
                System.out.println ("Fatal err loading properties file:"+e);
            }
        } else {
            tv = new DefaultTemplateView(node, "tid", "tdir");
        }

        //create the component and bind it to the view
        BTemplate templateComp = new BTemplate();
        templateComp.setView(tv);

        //now specify the model
        templateComp.addModel(new HelloWorldModel(useIDTemplate));
        String[] items = new String[] {"Cabbage", "Carrots", "Pickles", "Sugar", "Wheaties", "Flour", "Potatoes"};
        String[] qtys =  new String[] {"3 heads", "1 bag", "1 jar", "5 lbs", "3 bxs", "50 lbs", "25 lbs"};
        templateComp.addModel(new GroceriesModel(items, qtys));

        //add the component to the root
        root.addChild(templateComp);

        //return the page
        return page;
    }

    //--------------- GroceriesModel -------------------------
    /**
     * Groceries Model
     */
    class GroceriesModel extends AbstractTemplateModel implements IterativeModel {

        String[] items = null;
        String[] qtys = null;
        int cntr = -1;

        //constructor
        public GroceriesModel(String[] iitems, String[] iqtys) {
            items = iitems;
            qtys = iqtys;
        }

        //register the model by name
        public String getName() {return "Groceries";}

        //provide items by key
        public Object getItem(String key) {
            if (key.equals("Item")) return items[cntr];
            else if (key.equals("Qty")) return qtys[cntr];
            else return super.getItem(key);
        }

        //prepare for iteration
        public void preIterate() {
            cntr = -1;
        }

        //return true if there are more rows in the data model
        public boolean hasNext() {
            return (cntr<(items.length-1));
        }

        //load the next row in the model
        public void loadNext() {
            cntr++;
        }

        //cleanup after iteration
        public void postIterate() {}
    }

    //--------------- HelloWorldModel ------------------------
    /**
     * HelloWorldModel
     */
    class HelloWorldModel extends AbstractTemplateModel {

        String source = null;

        public HelloWorldModel(boolean useIDTemplate) {
            if (useIDTemplate) source = "Here the template processing instructions are stored in a separate .properties file and just referenced using the tid attribute.";
            else source = "Here the template processing instructions are actually stored in the template file as directives embedded in the tdir attributes.";
        }

        //register the model by name
        public String getName() {
            return "HelloWorld";
        }

        //provide items by key
        public Object getItem(String key) {
            if (key.equals("Title")) return "Hello World 3 example";
            else if (key.equals("Descr")) return "This example shows BTemplate processing an XML template. "+source;
            else if (key.equals("Hello")) return "Hello World! Hi Ma! Hi Pa!";
            else return super.getItem(key);
        }
    }
}
