/*
 * Copyright (C) 2003  Christian Cryder [christianc@granitepeaks.com]
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TestXmlcUtilsCreate.java 114 2005-12-09 15:51:51Z christianc $
 */
package org.barracudamvc.taskdefs;

import org.apache.log4j.Logger;

import org.enhydra.xml.xmlc.taskdef.*;

import org.barracudamvc.testbed.*;

/**
 * Unit tests for create() factory method of XmlcUtils
 */
public class TestXmlcUtilsCreate extends DefaultTestCase {

    //common vars (customize for every test class)
    private static String testClass = TestXmlcUtilsCreate.class.getName();
    private static Logger logger = Logger.getLogger("test."+testClass);

    //variables

    //-------------------- Basics --------------------------------
    /**
     * Public Constructor
     */
    public TestXmlcUtilsCreate(String name) {
        super(name);
    }

    /**
     * Every test class should have a main method so it can be run
     * directly (when debugging tests you often will not want to run
     * the whole suite)
     *
     * @param args defined in test.util.TestUtil
     */
    public static void main(String args[]) {
        //check for standard runtime parameters
        TestUtil.parseParams(args);

        //launch the test
        if (TestUtil.BATCH_MODE) junit.textui.TestRunner.main(new String[] {testClass});
        else junit.swingui.TestRunner.main(new String[] {testClass});
    }


    //-------------------- Actual Tests --------------------------
    //Note: all the methods herein should follow the testXXXX naming convention
    //Also keep in mind that local vars set in one test method are NOT retained
    //when the next method is invoked because JUnit makes a separate instance of
    //the test class for each testXXXX method!!!

    /**
     * Test class creation
     */
    public void testCreate() {
        if (logger.isInfoEnabled()) logger.info("testing create");

        XmlcUtils xmlcUtils = XmlcUtils.create();
        assertEquals("Default create should return a default implementation", DefaultXmlcUtilsImpl.class, xmlcUtils.getClass());

        System.setProperty(XmlcUtils.DEFAULT_UTILS_CLASS_KEY, String.class.getName());
        xmlcUtils = XmlcUtils.create();
        assertEquals("Improper create should return a default implementation", DefaultXmlcUtilsImpl.class, xmlcUtils.getClass());

        System.setProperty(XmlcUtils.DEFAULT_UTILS_CLASS_KEY, DummyXmlcUtils.class.getName() );
        xmlcUtils = XmlcUtils.create();
        assertEquals("Improper create should return dummy implementation", DummyXmlcUtils.class, xmlcUtils.getClass());
    }

}
